@@ -11,6 +11,7 @@ import (
 	"net/http"
 	"os"
 	"runtime"
+	"strconv"
 	"strings"
 
 	"github.com/gorilla/mux"
@@ -25,7 +26,7 @@ type Daemon struct {
 	tomb        tomb.Tomb
 	unixl       net.Listener
 	tcpl        net.Listener
-	idMap       *shared.Idmap
+	IdmapSet    *shared.IdmapSet
 	lxcpath     string
 	certf       string
 	keyf        string
@@ -163,6 +164,16 @@ func isJsonRequest(r *http.Request) bool {
 	return false
 }
 
+func (d *Daemon) isRecursionRequest(r *http.Request) bool {
+	recursion_str := r.FormValue("recursion")
+	recursion, err := strconv.Atoi(recursion_str)
+	if err != nil {
+		return false
+	}
+
+	return recursion == 1
+}
+
 func (d *Daemon) createCmd(version string, c Command) {
 	var uri string
 	if c.name == "" {
@@ -286,16 +297,15 @@ func StartDaemon(listenAddr string) (*Daemon, error) {
 		NotFound.Render(w)
 	})
 
-	d.idMap, err = shared.NewIdmap()
+	d.IdmapSet, err = shared.DefaultIdmapSet()
 	if err != nil {
 		shared.Logf("error reading idmap: %s", err.Error())
 		shared.Logf("operations requiring idmap will not be available")
 	} else {
-		shared.Debugf("idmap is %d %d %d %d\n",
-			d.idMap.Uidmin,
-			d.idMap.Uidrange,
-			d.idMap.Gidmin,
-			d.idMap.Gidrange)
+		shared.Debugf("Default uid/gid map:")
+		for _, lxcmap := range d.IdmapSet.ToLxcString() {
+			shared.Debugf(" - " + lxcmap)
+		}
 	}
 
 	localSocket := shared.VarPath("unix.socket")
