@@ -17,6 +17,7 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
+	"net"
 	"net/http"
 	"os"
 	"path"
@@ -125,6 +126,10 @@ void create_pipe(int *master, int *slave) {
 */
 import "C"
 
+const (
+	SYS_CLASS_NET = "/sys/class/net"
+)
+
 func OpenPty(uid, gid int) (master *os.File, slave *os.File, err error) {
 	fd_master := C.int(-1)
 	fd_slave := C.int(-1)
@@ -652,3 +657,17 @@ func ReadLastNLines(f *os.File, lines int) (string, error) {
 
 	return string(data), nil
 }
+
+func IsBridge(iface *net.Interface) bool {
+	p := path.Join(SYS_CLASS_NET, iface.Name, "bridge")
+	stat, err := os.Stat(p)
+	if err != nil {
+		return false
+	}
+
+	return stat.IsDir()
+}
+
+func IsLoopback(iface *net.Interface) bool {
+	return int(iface.Flags&net.FlagLoopback) > 0
+}
