@@ -99,6 +99,7 @@ cleanup() {
     [ -n "${LXD3_DIR}" ] && wipe "${LXD3_DIR}"
     [ -n "${LXD4_DIR}" ] && wipe "${LXD4_DIR}"
     [ -n "${LXD_MIGRATE_DIR}" ] && wipe "${LXD_MIGRATE_DIR}"
+    [ -n "${LXD_SERVERCONFIG_DIR}" ] && wipe "${LXD_SERVERCONFIG_DIR}"
 
     echo ""
     echo ""
@@ -114,6 +115,7 @@ fi
 . ./basic.sh
 . ./concurrent.sh
 . ./database.sh
+. ./deps.sh
 . ./fuidshift.sh
 . ./migration.sh
 . ./remote.sh
@@ -121,6 +123,7 @@ fi
 . ./snapshots.sh
 . ./static_analysis.sh
 . ./config.sh
+. ./serverconfig.sh
 . ./profiling.sh
 . ./fdleak.sh
 . ./database_update.sh
@@ -171,6 +174,9 @@ test_commits_signed_off
 echo "==> TEST: doing static analysis of commits"
 static_analysis
 
+echo "==> TEST: checking dependencies"
+test_check_deps
+
 echo "==> TEST: Database schema update"
 test_database_update
 
@@ -192,9 +198,15 @@ test_remote_usage
 echo "==> TEST: snapshots"
 test_snapshots
 
+echo "==> TEST: snapshot restore"
+test_snap_restore
+
 echo "==> TEST: profiles, devices and configuration"
 test_config_profiles
 
+echo "==> TEST: server config"
+test_server_config
+
 if type fuidshift >/dev/null 2>&1; then
     echo "==> TEST: uidshift"
     test_fuidshift
